/*
 * File:	bounce.cpp
 * Purpose:	wxMesa sample app
 * Author:	Julian Smart
 * Created:	1995
 * Updated:	Wolfram Gloger (for use with GLX)
 * Copyright:	(c) 1995
 */

static const char sccsid[] = "%W% %G%";

#ifdef __GNUG__
#pragma implementation
#pragma interface
#endif

#include "wx_setup.h"

#if defined(wx_msw)

#include "wx.h"
#include "wx_timer.h"
#include "wx_privt.h"
#include "wxmesa.h"

#elif defined(wx_motif)

#include "wx.h"
#include "wx_timer.h"
#include "wx_privt.h"
#include "gl_canvs.h"
#include <GL/gl.h>
#include <GL/glu.h>

#elif defined(wx_xt)

#define Uses_wxTimer
#define Uses_GLCanvas
#include "wx.h"

#endif

#include "bounce.h"

#include <math.h>

#define COS(X)   cos( (X) * 3.14159/180.0 )
#define SIN(X)   sin( (X) * 3.14159/180.0 )

GLuint Ball;
GLenum Mode;
GLfloat Zrot = 0.0, Zstep = 6.0;
GLfloat Xpos = 0.0, Ypos = 1.0;
GLfloat Xvel = 0.2, Yvel = 0.0;
GLfloat Xmin=-4.0, Xmax=4.0;
GLfloat Ymin=-4.0, Ymax=4.0;
GLfloat G = -0.1;

class TestTimer: public wxTimer
{
public:
  void Notify(void);
};

TestTimer timer;
MyFrame *frame = NULL;


// This statement initializes the whole application and calls OnInit
MyApp myApp;

// A macro needed for some compilers (AIX) that need 'main' to be defined
// in the application itself.
IMPLEMENT_WXWIN_MAIN

// `Main program' equivalent, creating windows and returning main app frame
wxFrame *MyApp::OnInit(void)
{
  // Create the main frame window
  frame = new MyFrame(NULL, "wxMesa Bounce Sample", 50, 50, 200, 200);

  // Give it an icon
#ifdef wx_msw
  frame->SetIcon(new wxIcon("mondrian"));
#endif
#ifdef wx_x
  frame->SetIcon(new wxIcon("aiai.xbm"));
#endif

  // Make a menubar
  wxMenu *file_menu = new wxMenu;

  file_menu->Append(MINIMAL_QUIT, "E&xit");
  wxMenuBar *menu_bar = new wxMenuBar;
  menu_bar->Append(file_menu, "&File");
  frame->SetMenuBar(menu_bar);

  // Make a panel with a message
  frame->canvas = new TestGLCanvas(frame, 0, 0, 200, 200);

  // Show the frame
  frame->Show(TRUE);

  timer.Start(100);

  // Return the main frame window
  return frame;
}

void TestTimer::Notify(void)
{
    if (frame && frame->canvas) {
	//cerr << "Drawing" << endl;
	frame->canvas->Test();
    }
}

// My frame constructor
MyFrame::MyFrame(wxFrame *frame, char *title, int x, int y, int w, int h):
  wxFrame(frame, title, x, y, w, h)
{}

// Intercept menu commands
void MyFrame::OnMenuCommand(int id)
{
  switch (id) {
    case MINIMAL_QUIT:
    {
      timer.Stop();
      if (OnClose())
        delete this;
      break;
    }
    break;
  }
}

/*
 * TestGLCanvas implementation
 */

TestGLCanvas::TestGLCanvas(wxWindow *parent, int x, int y, int w, int h, long style, char *name):
  GLCanvas(parent, x, y, w, h, style, name)
{
   parent->Show(TRUE);
   GLSetCurrent();
   Ball = make_ball();
   glCullFace( GL_BACK );
   glEnable( GL_CULL_FACE );
   glDisable( GL_DITHER );
   glShadeModel( GL_FLAT );
}

TestGLCanvas::~TestGLCanvas(void)
{
}

GLuint TestGLCanvas::make_ball( void )
{
   GLuint list;
   GLfloat a, b;
   GLfloat da = 18.0, db = 18.0;
   GLfloat radius = 1.0;
   GLuint color;
   GLfloat x, y, z;

   list = glGenLists( 1 );

   glNewList( list, GL_COMPILE );

   color = 0;
   for (a=-90.0;a+da<=90.0;a+=da) {

      glBegin( GL_QUAD_STRIP );
      for (b=0.0;b<=360.0;b+=db) {

	 if (color)
	    GLSetColour("RED");
	 else
	    GLSetColour("WHITE");

	 x = COS(b) * COS(a);
	 y = SIN(b) * COS(a);
	 z = SIN(a);
	 glVertex3f( x, y, z );

	 x = radius * COS(b) * COS(a+da);
	 y = radius * SIN(b) * COS(a+da);
	 z = radius * SIN(a+da);
	 glVertex3f( x, y, z );

	 color = 1-color;
      }
      glEnd();

   }

   glEndList();

   return list;
}


void TestGLCanvas::Test( void )
{
  static Bool guard = FALSE;

  if (guard)
    return;
    
  guard = TRUE;

  static float vel0 = -100.0;

  Zrot += Zstep;

  Xpos += Xvel;
  if (Xpos>=Xmax) {
      Xpos = Xmax;
      Xvel = -Xvel;
      Zstep = -Zstep;
   }
  if (Xpos<=Xmin) {
      Xpos = Xmin;
      Xvel = -Xvel;
      Zstep = -Zstep;
   }

  Ypos += Yvel;
  Yvel += G;
  if (Ypos<Ymin) {
      Ypos = Ymin;
      if (vel0==-100.0)  vel0 = fabs(Yvel);
      Yvel = vel0;
   }
#ifdef wx_msw
#if (wxMAJOR_VERSION < 2) && (wxMINOR_VERSION <= 6) && (wxRELEASE_NUMBER <=3)
  InvalidateRect(GetHWND(),NULL,0);
#else
  // Refresh has arguments from 1.64 onwards.
  Refresh(FALSE);
#endif
#else
  OnPaint();
#endif /* wx_msw */

  guard = FALSE;
}

void TestGLCanvas::OnPaint( void )
{
   GLSetCurrent();

   glClear( GL_COLOR_BUFFER_BIT );

   glPushMatrix();
   glTranslatef( Xpos, Ypos, 0.0 );
   glScalef( 2.0, 2.0, 2.0 );
   glRotatef( 8.0, 0.0, 0.0, 1.0 );
   glRotatef( 90.0, 1.0, 0.0, 0.0 );
   glRotatef( Zrot, 0.0, 0.0, 1.0 );

   int err = glGetError();
   if(err != GL_NO_ERROR)
       cerr << "a gl Error: " << hex << err << endl;
   glCallList( Ball );
   err = glGetError();
   if(err != GL_NO_ERROR)
       cerr << "b gl Error: " << hex << err << " list=" << Ball << endl;

   glPopMatrix();
   glFlush();
   GLSwapBuffers();
}

void TestGLCanvas::OnSize(int width, int height)
{
#ifdef wx_msw
  if (glContext)
  {
#endif
    GLSetCurrent();
    glViewport(0, 0, (GLint)width, (GLint)height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho( -6.0, 6.0, -6.0, 6.0, -6.0, 6.0 );
    glMatrixMode(GL_MODELVIEW);
#ifdef wx_msw
  }
#endif
}
