/*								-*- C++ -*-
 * $Id: demo.h,v 1.2 1996-11-08 13:56:31+01 mho Exp $
 *
 * Purpose: demo for wxWindows Xt port inclusive demo for wxCanvas
 *
 * Authors: Markus Holzem and Julian Smart
 *
 * Copyright: (C) 1995, AIAI, University of Edinburgh (Julian)
 * Copyright: (C) 1995, GNU (Markus)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef demo_h
#define demo_h

#ifdef __GNUG__
#pragma interface
#endif

//-----------------------------------------------------------------------------
// derived classes
//-----------------------------------------------------------------------------

class MyApp;
class MyFrame;
class MyPanel;
class MyTransient;
class MyCanvas;

class MyApp : public wxApp {
public:
    MyApp(void) : wxApp() { wx_class = "Test"; };

    wxFrame* OnInit(void);
    int      OnExit(void);
};

class HelloFrame : public wxFrame {
public:
    HelloFrame(wxFrame *frame, char *title, int x=-1, int y=-1, int w=-1, int h=-1)
	: wxFrame(frame, title, x, y, w, h, wxDEFAULT_FRAME, "HelloFrame")
    { }

    void  OnMenuCommand(int id);
    void  OnMenuSelect(int id);

    Bool  OnClose(void);
    void  OnSize(int width, int height);

    void  Draw(wxDC& dc, Bool on_canvas=TRUE);
};

class HelloPanel : public wxPanel {
public:
    HelloPanel(wxFrame *parent, int x=-1, int y=-1,
	       int width=-1, int height=-1, int style=wxNO_DC|wxBORDER,
	       char *name="HelloPanel")
	: wxPanel(parent, x, y, width, height, style, name) {}

    void OnCommand(wxWindow &win, wxCommandEvent &event);
    void OnDefaultAction(wxItem *item);

    void OnItemLeftClick(wxItem *item, int x, int y, int keys);
    void OnItemRightClick(wxItem *item, int x, int y, int keys);
    void OnItemSelect(wxItem *item, Bool select);
};

class MyCanvas : public wxCanvas {
public:
    MyCanvas(wxFrame *parent, int x=-1, int y=-1, int width=-1, int height=-1,
	     int style=0, char *name="MyCanvas")
	: wxCanvas(parent, x, y, width, height, style, name)
    {
	popup = NULL;
	alpha1 = 0.0;
	alpha2 = 90.0;
    }
    ~MyCanvas(void)
    {
	delete popup;
    }

    void OnChar(wxKeyEvent& event);
    void OnEvent(wxMouseEvent& event);
    void OnPaint(void);

    // popup menu child
    wxMenu *popup;
    float alpha1, alpha2;
};

#if USE_XT_TOOLBAR
#   define TBAR_STYLE (wxNO_DC | wxPRIVATE_CHILD)
#else
#   define TBAR_STYLE (wxPRIVATE_CHILD)
#endif

class MyToolBar : public wxToolBar {
public:
    MyToolBar(wxFrame *frame, int x = 0, int y = 0, int w = -1, int h = -1,
	      long style = TBAR_STYLE,
	      int direction = wxHORIZONTAL, int RowsOrColumns = 30)
	: wxToolBar(frame, x, y, w, h, style, direction, RowsOrColumns)
    {}

    Bool OnLeftClick(int toolIndex, Bool toggled);
    void OnMouseEnter(int toolIndex);
};

class MyTimer : public wxTimer {
public:
    void Notify(void);
};

//-----------------------------------------------------------------------------
// constants for menus
//-----------------------------------------------------------------------------

#define TEST_FILE     			1000
#define TEST_NEW			1001
#define TEST_LOAD     			1002
#define TEST_SAVE     			1003
#define TEST_SAVEAS   			1004
#define TEST_PRINTEPS 			1005
#define TEST_QUIT     			1006

#define TEST_TIMER			2000
#define TEST_TIMERON    		2001
#define TEST_TIMEROFF   		2002
#define TEST_CHECKABLE  		2004

#define TEST_CANVAS			3000

#define TEST_SCALE			3100
#define TEST_TWIPS      		3101
#define TEST_LOMETRIC   		3102
#define TEST_METRIC     		3103
#define TEST_POINT      		3104
#define TEST_NORMAL     		3105
#define TEST_ZOOM       		3106

#define TEST_CURSOR			3200
#define TEST_CURSOR_ARROW		3201
#define TEST_CURSOR_BASED_ARROW_DOWN	3202
#define TEST_CURSOR_BASED_ARROW_UP	3203
#define TEST_CURSOR_BULLSEYE		3204
#define TEST_CURSOR_CROSS		3205
#define TEST_CURSOR_CROSS_REVERSE	3206
#define TEST_CURSOR_DOUBLE_ARROW	3207
#define TEST_CURSOR_HAND		3208
#define TEST_CURSOR_IBEAM		3209
#define TEST_CURSOR_LEFT_BUTTON		3210
#define TEST_CURSOR_MAGNIFIER		3211
#define TEST_CURSOR_MIDDLE_BUTTON	3212
#define TEST_CURSOR_NO_ENTRY		3213
#define TEST_CURSOR_PAINT_BRUSH		3214
#define TEST_CURSOR_PENCIL		3215
#define TEST_CURSOR_POINT_LEFT		3216
#define TEST_CURSOR_POINT_RIGHT		3217
#define TEST_CURSOR_QUESTION_ARROW	3218
#define TEST_CURSOR_RIGHT_BUTTON	3219
#define TEST_CURSOR_SIZENESW		3220
#define TEST_CURSOR_SIZENS		3221
#define TEST_CURSOR_SIZENWSE		3222
#define TEST_CURSOR_SIZEWE		3223
#define TEST_CURSOR_SIZING		3224
#define TEST_CURSOR_SPRAYCAN		3225
#define TEST_CURSOR_WAIT		3226
#define TEST_CURSOR_WATCH		3227
#define TEST_CURSOR_CHAR		3228
#define TEST_CURSOR_BLANK		3229

#define TEST_OPTIONS			4000
#define TEST_CHANGE_FONT		4001
#define TEST_CHANGE_COLOUR		4002

#define TEST_HELP			5000
#define TEST_ABOUT    			5001

#define TEST_POPUP1			9000
#define TEST_POPUP2			9001
#define TEST_POPUP3			9002

#define TAG_BUTTON			10000
#define TAG_MOVE_TBAR			10001
#define TAG_SINGLETEXT			10002
#define TAG_CHECK			10003
#define TAG_CHOICE			10004
#define TAG_RADIO			10005
#define TAG_LIST			10006
#define TAG_MULTITEXT			10007
#define TAG_MESSAGE			10008
#define TAG_SLIDER			10009

#define TEST_TB_LOAD			0
#define TEST_TB_SAVEAS			1
#define TEST_TB_CUT			2
#define TEST_TB_COPY			3
#define TEST_TB_PASTE			4
#define TEST_TB_UNDO			5
#define TEST_TB_DEBUG			6
#define TEST_TB_EMODE			7
#define TEST_TB_SCALE			8

#endif // demo_h
