/* -*- C++ -*-
 * File:	threads.cpp
 * Purpose:	Sample for wxWindows threads
 * Author:	Wolfram Gloger
 * Created:	1996
 * Updated:	
 */

#ifdef __GNUG__
#pragma implementation "threads.h"
#endif

#include "wx_setup.h"

#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#ifdef wx_xt
#define Uses_wxTimer
#include "wx.h"
#else
#include "wx.h"
#include "wx_timer.h"
#endif

#include "threads.h"

#define I_MAX          500
#define MAX_USEC       200
#define USE_BITMAP     1
#define TIMER_INTERVAL 3000 /* 3 seconds */

enum MyFrame_menuid {
	MENU_TEST = 0,
	MENU_START,
	MENU_STOP,
	MENU_NEW,
	MENU_NEW4,
	MENU_DEL,
	MENU_DEL_ALL,
	MENU_QUIT
};

class
MyThread: public wxThread
{
public:
	MyThread(wxTextWindow *tw, wxCanvas* cv): wxThread(), m_count(0) {
		text_window = tw;
		canvas = cv;
		buf = new unsigned [buf_size];
	}
	~MyThread() {
		Cancel();
		Join();
		if(m_count != 0) wxDebugMsg("Bug in thread class!\n");
		delete[] buf;
	}
	void* Entry();
	void OnExit();
private:
	wxTextWindow *text_window;
	wxCanvas* canvas;
	unsigned* buf;
	static const long buf_size;
	int m_count;
};

const long MyThread::buf_size = (100*100)/sizeof(unsigned);

void*
MyThread::Entry()
{
	long id, i, r;
	struct timeval tv;

	id = GetID();
	for(i=0, r=id; i<buf_size; i++) {
		r = r*534377l % 2233177l;
		buf[i] = r;
	}
	for(i=0; i<I_MAX; i++) {
		TestCancel();
		m_count++;
		wxMainMutex.Lock();
#if USE_BITMAP
		canvas->DrawBitmap8((unsigned char*)buf, 100, 100, 0, 0.0, 0.0);
#endif
		if(text_window->GetInsertionPoint() > 20000)
			text_window->Clear();
		(*text_window) << "thread=" << id << " count=" << i << "\n";
		text_window->SetInsertionPointEnd();
		r = random()%1000;
		m_count--;
		wxMainMutex.Unlock();
		tv.tv_sec = 0;
		tv.tv_usec = r*MAX_USEC;
		select(0, 0, 0, 0, &tv);
	}
	return 0;
}

// This is always called with wxMainMutex locked.
void
MyThread::OnExit()
{
	Join();
	(*text_window) << "thread=" << (long)GetID() << " has exited\n";
	text_window->SetInsertionPointEnd();
}

MyFrame *main_frame;

// This timer continuously cancels and re-starts a thread.
class
MyTimer: public wxTimer
{
public:
	MyTimer(): wxTimer(), thr(0) {}
	~MyTimer() {
		if(thr) delete thr;
	}
	void Notify() {
		if(!thr) thr = main_frame->NewThread();
		if(thr) {
			thr->Cancel();
			thr->Join();
			thr->Create();
		}
	}
private:
	wxThread* thr;
} * main_timer;

// This statement initialises the whole application
MyApp myApp;

MyApp::MyApp()
{
}

// The `main program' equivalent, creating the windows and returning the
// main frame
wxFrame *MyApp::OnInit(void)
{
	// Create the main frame window
	main_frame = new MyFrame(NULL, "Threads demo", 0, 0, 400, 500);

	// Make a menubar
	wxMenu *file_menu = new wxMenu;

	file_menu->Append(MENU_NEW, "New thread");
	file_menu->Append(MENU_NEW4, "4 new threads");
	file_menu->Append(MENU_DEL, "Delete thread");
	file_menu->Append(MENU_DEL_ALL, "Delete all threads");
	file_menu->Append(MENU_START, "Start thread");
	file_menu->Append(MENU_STOP, "Cancel thread");
	file_menu->Append(MENU_TEST, "Test Menu");
	file_menu->AppendSeparator();
	file_menu->Append(MENU_QUIT, "Quit");

	wxMenuBar *menu_bar = new wxMenuBar;

	menu_bar->Append(file_menu, "File");

	// Associate the menu bar with the frame
	main_frame->SetMenuBar(menu_bar);

	return main_frame;
}

// Define my frame constructor
MyFrame::MyFrame(wxFrame *frame, char *title, int x, int y, int w, int h):
wxFrame(frame, title, x, y, w, h)
{
	thr_i = 0;
	for(int i=0; i<10; i++) thr[i] = 0;
	text_window = new wxTextWindow(this, 0, 0, 350, 250, wxNATIVE_IMPL);
#if USE_BITMAP
	canvas = new wxCanvas(this, 0, 250, 100, 100);
#else
	canvas = 0;
#endif

	// Give it a status line
	CreateStatusLine();

	Fit();
	Show(TRUE);

	// start timer
	main_timer = new MyTimer();
	main_timer->Start(TIMER_INTERVAL);
}

MyFrame::~MyFrame()
{
	delete main_timer;

	while(thr_i > 0) {
		delete thr[--thr_i];
	}
	delete canvas;
	delete text_window;
}

// Allocate a new thread and start it
wxThread*
MyFrame::NewThread()
{
	MyThread* thr;

	thr = new MyThread(text_window, canvas);
	thr->Create();
	return thr;
}

// Intercept menu commands
void
MyFrame::OnMenuCommand(int id)
{
	char buf[80];

	switch (id) {
	case MENU_TEST:
        wxMessageBox("Message", "This is a message", wxOK|wxCENTRE);
		break;
    case MENU_QUIT:
		Close();
		break;
	case MENU_NEW:
		if(thr_i < 10) {
			thr[thr_i++] = NewThread();
			sprintf(buf, "Number of threads: %d", thr_i+1);
			SetStatusText(buf);
		}
		break;
	case MENU_NEW4:
		if(thr_i < 6) {
			thr[thr_i++] = NewThread();
			thr[thr_i++] = NewThread();
			thr[thr_i++] = NewThread();
			thr[thr_i++] = NewThread();
			sprintf(buf, "Number of threads: %d", thr_i+1);
			SetStatusText(buf);
		}
		break;
	case MENU_DEL:
		if(thr_i > 0) {
			wxPostDelete(thr[--thr_i]);
			sprintf(buf, "Number of threads: %d", thr_i+1);
			SetStatusText(buf);
		}
		break;
	case MENU_DEL_ALL:
		while(thr_i > 0) {
			wxPostDelete(thr[--thr_i]);
		}
		sprintf(buf, "Number of threads: %d", thr_i+1);
		SetStatusText(buf);
		break;
	case MENU_START:
		if(thr_i > 0) {
			thr[thr_i-1]->Create();
			sprintf(buf, "Start thread %d", thr_i-1);
			SetStatusText(buf);
		}
		break;
	case MENU_STOP:
		if(thr_i > 0) {
			thr[thr_i-1]->Cancel();
			thr[thr_i-1]->Join();
			sprintf(buf, "Stop thread %d", thr_i-1);
			SetStatusText(buf);
		}
		break;
	}
}

Bool
MyFrame::OnClose(void)
{
	Show(FALSE);

	return TRUE;
}

/*
 * Local variables:
 * tab-width: 4
 * End:
 */
