/* -*- C++ -*-
 * File:	wxtiff.cpp
 * Purpose:	TIFF viewer for wxWindows class library
 * Author:	Wolfram Gloger
 * Created:	1996
 * Updated:	
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "wx.h"

#include "wxtiff.h"

// This statement initialises the whole application
MyApp myApp;

MyApp::MyApp()
{
}

// The `main program' equivalent, creating the windows and returning the
// main frame
wxFrame *MyApp::OnInit(void)
{
	// Create the main frame window
	MyFrame *frame = new MyFrame(NULL, "wxTIFF", 0, 0, 500, 300);

	// Give it a status line
	// frame->CreateStatusLine(2);

	// Make a menubar
	wxMenu *file_menu = new wxMenu;

	file_menu->Append(MENU_OPEN_TIFF, "Open TIFF");
	file_menu->AppendSeparator();
	file_menu->Append(MENU_QUIT, "Quit");

	wxMenuBar *menu_bar = new wxMenuBar;

	menu_bar->Append(file_menu, "File");

	// Associate the menu bar with the frame
	frame->SetMenuBar(menu_bar);

	// Return the main frame window
	return frame;
}

// Define my frame constructor
MyFrame::MyFrame(wxFrame *frame, char *title, int x, int y, int w, int h):
wxFrame(frame, title, x, y, w, h)
{
	slot_list = new wxList(wxKEY_STRING);
	panel = new wxPanel(this, 0, 0, 400, 200);

	panel->SetLabelPosition(wxVERTICAL) ;

	list_box = new wxListBox(panel, NULL, "Images", wxSINGLE|wxALWAYS_SB,
							 -1, -1, 400, 200);
	panel->NewLine();
	panel->Fit();
	Fit();
	Show(TRUE);
}

MyFrame::~MyFrame()
{
	delete slot_list;
}

// Intercept menu commands
void
MyFrame::OnMenuCommand(int id)
{
	char *s;

	switch (id) {
	case MENU_OPEN_TIFF:
		s = wxFileSelector("Load TIFF file", NULL, NULL, NULL, "*.tif");
		if (s) {
			Load(s);
		}
		break;
    case MENU_QUIT:
		if(OnClose()) wxPostDelete(this);
		break;
	}
}

void
MyFrame::OnSize(int, int)
{
	if (panel) {
		int width, height;
		panel->GetClientSize(&width, &height);
		list_box->SetSize(0, 0, width, height);
		panel->Fit();
	}
}

void
MyFrame::Rename(TiffFrame *tf, char *new_name)
{
	wxNode *node = slot_list->Member(tf);
	if(!node) {
		wxDebugMsg("Trying to rename unknown subframe!\n");
		return;
	}
	// Update the listbox.
	int i = list_box->FindString(tf->GetFileName());
	if(i < 0) {
		wxDebugMsg("Inconsistent choice list (item=%s)\n", tf->GetFileName());
		return;
	}
	list_box->Delete(i);
	list_box->Append(new_name);
}

void
MyFrame::Load(char *title)
{
	TiffFrame *tf = new TiffFrame(this, title, title, -1, -1, -1, -1);
	slot_list->Append(title, tf);
	list_box->Append(title);
}

void
MyFrame::Delete(TiffFrame *tf)
{
	wxNode *node = slot_list->Member(tf);
	if(!node) {
		wxDebugMsg("Trying to delete unknown subframe!\n");
		return;
	}
	// Update the choices lists.
	int i = list_box->FindString(tf->GetFileName());
	if(i < 0) {
		wxDebugMsg("Inconsistent choice list (item=%s)\n", tf->GetFileName());
		return;
	}
	list_box->Delete(i);
	slot_list->DeleteNode(node);
	wxPostDelete(tf);
}

Bool
MyFrame::OnClose(void)
{
	Show(FALSE);

	return TRUE;
}

/*
 * Local variables:
 * tab-width: 4
 * End:
 */
