/* -*- C++ -*-
 * $Id: APP_thread.h,v 1.3 1997/05/01 21:34:46 wg Exp $
 *
 * Purpose:   Basic thread support for wxWindows.
 * Author:    Wolfram Gloger
 * Copyright: (C) 1996, 1997 Wolfram Gloger
 * License:   GNU Library General Public License
 */

#ifndef wx_thrd_h
#define wx_thrd_h

#ifdef __GNUG__
#pragma interface
#endif

#if USE_THREADS_POSIX

#include <pthread.h>

#elif USE_THREADS_SGI

#include <abi_mutex.h>

#endif

// Mutual exclusion type.

class
wxMutex
{
public:
	wxMutex();
	~wxMutex();
	int Lock() {
#if USE_THREADS_POSIX
		return pthread_mutex_lock(&mutex);
#elif USE_THREADS_SGI
		spin_lock(&mutex);
		return 0;
#else
		return 0;
#endif
	}
	int TryLock() {
#if USE_THREADS_POSIX
		return pthread_mutex_trylock(&mutex);
#elif USE_THREADS_SGI
		return acquire_lock(&mutex);
#else
		return 0;
#endif
	}
	int Unlock() {
#if USE_THREADS_POSIX
		return pthread_mutex_unlock(&mutex);
#elif USE_THREADS_SGI
		return release_lock(&mutex);
#else
		return 0;
#endif
	}
private:
#if USE_THREADS_POSIX
	pthread_mutex_t mutex;
#elif USE_THREADS_SGI
	abilock_t mutex;
#endif
};

class wxThreadPrivate;

// The thread class.  Add your own functionality by overriding Entry().

class
wxThread: public wxObject
{
public:
	wxThread();
	virtual ~wxThread();
	int           Create(); // start the thread
	void*         Join();   // wait for thread termination; obtain result
	int           Cancel(); // request thread termination
	unsigned long GetID();  // get id of running thread
	virtual void  OnExit(); // this is called from wxApp::MainLoop()
protected:
	// these can only be called from _inside_ the running thread
	void Exit(void* status = 0);
	void TestCancel();
private:
	virtual void* Entry() = 0;
	friend class wxThreadPrivate;
	wxThreadPrivate *p;
};

Bool wxThreadIsMain();      // returns true if this is the main thread

// Global variables
extern wxMutex wxMainMutex;

#endif /* wx_thrd_h */

/*
 * Local variables:
 * tab-width: 4
 * End:
 */
